@@ -1,10 +1,21 @@
+"""Provide reference analytic noise shapes, in TDI and sensitivity. """
+
 import os
 import numpy as np
 import lisaconstants
 from scipy.interpolate import InterpolatedUnivariateSpline as spline
+from ldc.common.series import tdi
+from ldc.lisa.orbits import Orbits
+import lisaorbits
+
+#pylint:disable=C0103
+
 
 CLIGHT = lisaconstants.SPEED_OF_LIGHT
 YEAR = lisaconstants.SIDEREALYEAR_J2000DAY*24*60*60
+D2P = {12: '3', 23: '1', 31: '2', 13: '2p', 32: '1p', 21: '3p'}
+P2D = {'3': 12, '1': 23, '2': 31, '2p': 13, '1p': 32, '3p': 21}
+LINKS = [12, 23, 31, 13, 32, 21] #receiver/emitter
 
 known_noise_config = ["Proposal", "SciRDv1", "MRDv1", "MRD_MFR", "mldc",
                       "SciRDdeg1", "SciRDdeg2", "newdrs", "sangria", "spritz"]
@@ -56,9 +67,10 @@ def get_noise_model(model, frq=None, **kwargs):
         NoiseClass = globals()[model+"Noise"]
         return NoiseClass(frq, **kwargs)
 
+
 def _XY2AET(X, XY, option='A'):
     """ Convert TDI X and XY in TDI A, E, T.
-    
+
     """
     if option in ["A", "E"]:
         return X-XY # = E
@@ -67,9 +79,12 @@ def _XY2AET(X, XY, option='A'):
     else:
         raise NotImplementedError
 
-class Noise():
 
-    def __init__(self, frq, wd=0):
+class Noise():
+    """ Define analytic noise shape in TDI and sensitivity.
+    """
+
+    def __init__(self, frq, wd=0, orbits=None, t_obs=None):
         """Noise is defined for a given freq range.
 
         wd option add white dwarf confusion noise contribution, and is
@@ -77,7 +92,37 @@ class Noise():
         """
         self.freq = frq
         self.wd = wd
+        self.orbits = orbits
         self._arm_length = 2.5e9
+        self.t_obs = t_obs
+        self.orbit_t0 = 0.0
+        self.ltt = {}
+        self.set_arm_lengths()
+
+    def get_time_vector(self):
+        if self.t_obs is None:
+            self.t_obs = self.orbits.size * self.orbits.dt
+        self.orbit_t0 = self.orbits.t0
+        t_vect = np.arange(self.orbit_t0, self.orbits.t0 + self.t_obs, self.orbits.dt)
+        return t_vect
+
+    def set_arm_lengths(self):
+        """Compute the average arm lengths from the orbits
+        """
+
+        if self.orbits is None:
+            for link in LINKS:
+                self.ltt[link] = self._arm_length / CLIGHT
+        elif isinstance(self.orbits, Orbits):
+            t_vect = self.get_time_vector()
+            self.ltt = {link: np.mean(self.orbits.compute_travel_time(
+                str(link)[1], str(link)[0], t_vect).flatten()) for link in LINKS}
+        elif isinstance(self.orbits, lisaorbits.Orbits):
+            t_vect = self.get_time_vector()
+            self.ltt = {link: np.mean(self.orbits.compute_ltt(t_vect, link).flatten())
+                        for link in LINKS}
+        else:
+            raise TypeError("Unknown orbit type")
 
     def set_wdconfusion(self, duration):
         """ Set the white dwarf noise contribution in years.
@@ -89,12 +134,16 @@ class Noise():
 
     @property
     def arm_length(self):
+        """ LISA armlength.
+        """
         return self._arm_length
 
     @arm_length.setter
     def arm_length(self, value):
+        """ Set LISA armlength.
+        """
         self._arm_length = value
-    
+
     def wd_confusion(self, freq=None, option="X", tdi2=False):
         """ Compute the white dwarf confusion noise for PSD.
 
@@ -199,52 +248,69 @@ class AnalyticNoise(Noise):
     acceleration noise and optical metrology system (OMS) noise
     """
 
-    DSoms_d = {'Proposal':(10.e-12)**2, 'SciRDv1':(15.e-12)**2,
-               'MRDv1':(10.e-12)**2, 'MRD_MFR':(13.5e-12)**2,
-               'sangria':(7.9e-12)**2, 'spritz':(7.9e-12)**2}  # m^2/Hz
-    DSa_a = {'Proposal':(3.e-15)**2, 'SciRDv1':(3.e-15)**2,
-             'MRDv1':(2.4e-15)**2,'MRD_MFR':(2.7e-15)**2,
-             'sangria':(2.4e-15)**2, 'spritz':(2.4e-15)**2} # m^2/sec^4/Hz
-    
-    def __init__(self, frq, model="SciRDv1", wd=0, oms=None, acc=None):
+    psd_oms_d = {'Proposal':(10.e-12)**2, 'SciRDv1':(15.e-12)**2,
+                 'MRDv1':(10.e-12)**2, 'MRD_MFR':(13.5e-12)**2,
+                 'sangria':(7.9e-12)**2, 'spritz':(7.9e-12)**2}  # m^2/Hz
+    psd_a_a = {'Proposal':(3.e-15)**2, 'SciRDv1':(3.e-15)**2,
+               'MRDv1':(2.4e-15)**2,'MRD_MFR':(2.7e-15)**2,
+               'sangria':(2.4e-15)**2, 'spritz':(2.4e-15)**2} # m^2/sec^4/Hz
+    psd_mu_d = {'Proposal':0, 'SciRDv1':0,
+                'MRDv1':0,'MRD_MFR':0,
+                'sangria':0, 'spritz':(3e-12)**2} # m^2/Hz
+
+    def __init__(self, frq, model="SciRDv1", oms=None, acc=None, **kwargs):
         """Set two components noise contributions wrt given model.
         """
-        Noise.__init__(self, frq, wd=wd)
+        Noise.__init__(self, frq, **kwargs)
 
         # Use custom values
-        if model not in self.DSoms_d and oms is not None:
-            AnalyticNoise.DSoms_d[model] = oms
-        if model not in self.DSa_a and acc is not None:
-            AnalyticNoise.DSa_a[model] = acc
+        if oms is not None:
+            AnalyticNoise.psd_oms_d[model] = oms
+        if acc is not None:
+            AnalyticNoise.psd_a_a[model] = acc
         self.model = model
+        self.oms_relaxation = not(model=='spritz')
         self.set_freq(frq)
-            
+
     def set_freq(self, frq):
+        """Set frequency array and compute the phasemeter
+        TM and OMS noises
+
+        Parameters
+        ----------
+        frq : ndarray
+            frequency array
+        """
 
         self.freq = frq
-        
+
         # Acceleration noise
-        Sa_a = AnalyticNoise.DSa_a[self.model] * (1.0 +(0.4e-3/frq)**2) *\
+        sa_a = AnalyticNoise.psd_a_a[self.model] * (1.0 +(0.4e-3/frq)**2) *\
                (1.0+(frq/8e-3)**4) # in acceleration
-        self.Sa_d = Sa_a*(2.*np.pi*frq)**(-4.) # in displacement
-        Sa_nu = self.Sa_d*(2.0*np.pi*frq/CLIGHT)**2 # in rel freq unit
-        self.Spm =  Sa_nu
+        self.sa_d = sa_a*(2.*np.pi*frq)**(-4.) # in displacement
+        sa_nu = self.sa_d*(2.0*np.pi*frq/CLIGHT)**2 # in rel freq unit
+        self.s_pm = sa_nu
 
         # Optical Metrology System
-        self.Soms_d = AnalyticNoise.DSoms_d[self.model] * (1. + (2.e-3/frq)**4) # in displacement
-        Soms_nu = self.Soms_d*(2.0*np.pi*frq/CLIGHT)**2 # in rel freq unit
-        self.Sop =  Soms_nu
+        relax = (1. + (2.e-3/frq)**4) if self.oms_relaxation else 1.0
+        self.psd_oms_d = AnalyticNoise.psd_oms_d[self.model] * relax # in displacement
+        s_oms_nu = self.psd_oms_d*(2.0*np.pi*frq/CLIGHT)**2 # in rel freq unit
+        self.s_op =  s_oms_nu
 
+        # Backlink
+        self.s_mu = AnalyticNoise.psd_mu_d[self.model] * (1. + (2.e-3/frq)**4)
+        # Convert into fractional frequency
+        self.s_mu *= (2 * np.pi * frq / CLIGHT) ** 2
 
     def relative_freq(self):
         """ Return acceleration and OMS noise in relative freq unit
         """
-        return self.Spm, self.Sop
+        return self.s_pm, self.s_op
 
     def displacement(self):
         """ Return acceleration and OMS noise in displacement
         """
-        return self.Sa_d, self.Soms_d
+        return self.sa_d, self.psd_oms_d
 
     def sensitivity(self):
         """ Return sensitivity.
@@ -254,7 +320,7 @@ class AnalyticNoise(Noise):
         array([3.94498567e-27, 1.53147971e-34, 5.43687367e-40, 1.53337371e-39,
                4.07338302e-37])
         """
-        all_m = np.sqrt(4.0*self.Sa_d + self.Soms_d)
+        all_m = np.sqrt(4.0*self.sa_d + self.psd_oms_d)
         av_resp = np.sqrt(5) # average the antenna response
         proj = 2./np.sqrt(3) # projection effect
 
@@ -268,7 +334,22 @@ class AnalyticNoise(Noise):
             sens += GalNoise(self.freq, self.wd*YEAR).galshape()
         return sens
 
-    def psd(self, freq=None, option="X", tdi2=False):#, includewd=0):
+    def _single_link_cov(self):
+        """ Return single link covariance
+        """
+        eta_mat_tm = tdi.eta_matrix(self.freq, self.ltt, noise_type='TM')
+        eta_mat_op = tdi.eta_matrix(self.freq, self.ltt, noise_type='SCI')
+        eta_mat_mu = tdi.eta_matrix(self.freq, self.ltt, noise_type='REF') + eta_mat_tm
+        eta_cor_tm = tdi.dot(eta_mat_tm, np.swapaxes(eta_mat_tm, 1, 2).conj())
+        eta_cor_op = tdi.dot(eta_mat_op, np.swapaxes(eta_mat_op, 1, 2).conj())
+        eta_cor_mu = tdi.dot(eta_mat_mu, np.swapaxes(eta_mat_mu, 1, 2).conj())
+        cov_eta_tm = eta_cor_tm * self.s_pm[:, np.newaxis, np.newaxis]
+        cov_eta_op = eta_cor_op * self.s_op[:, np.newaxis, np.newaxis]
+        cov_eta_mu = eta_cor_mu * self.s_mu[:, np.newaxis, np.newaxis]
+        cov_eta = 4 * cov_eta_tm + cov_eta_op + cov_eta_mu
+        return cov_eta
+
+    def psd(self, freq=None, option="X", tdi2=False, equal_arms=True):
         """ Return noise PSD at given freq. or freq. range.
 
         Option can be X, XY, A, E, T.
@@ -278,48 +359,76 @@ class AnalyticNoise(Noise):
         array([7.13597299e-40, 2.76990908e-42, 9.52379492e-43, 1.92645601e-40,
                1.15359813e-36])
         """
-        if option in ['A', 'E', 'T']:
-            XX = self.psd(freq, option="X", tdi2=tdi2)
-            XY = self.psd(freq, option="XY", tdi2=tdi2)
-            return _XY2AET(XX, XY, option)
+        # print("Option: " + option)
+        # if option in ['A', 'E', 'T']:
+        #     XX = self.psd(freq, option="X", tdi2=tdi2)
+        #     XY = self.psd(freq, option="XY", tdi2=tdi2)
+        #     return _XY2AET(XX, XY, option)
 
         if (self.wd and option in ["T"]):
             raise NotImplementedError
 
         if freq is not None:
-            self.__init__(freq, model=self.model, wd=self.wd)
+            self.set_freq(freq)
+
+        # Light travel time
+        lisa_lt = self.arm_length/CLIGHT
+        # Angular frequency
+        omega = 2.0 * np.pi * self.freq
+        if equal_arms:
+            x = omega * lisa_lt
+            if option=="X":
+                s_n = 16.0 * np.sin(x)**2 * (2.0 * (1.0 + np.cos(x)**2) * self.s_pm +\
+                                             self.s_op)
+            elif option=="XY":
+                s_n = -4.0 * np.sin(2*x) * np.sin(x) * (self.s_op + 4.0*self.s_pm)
+            elif option in ["A", "E"]:
+                s_n = 8.0 * np.sin(x)**2 * (2.0 * self.s_pm * (3.0 + 2.0*np.cos(x) +\
+                                                               np.cos(2*x)) +\
+                                            self.s_op * (2.0 + np.cos(x)))
+            elif option=="T":
+                s_n = 16.0 * self.s_op * (1.0 - np.cos(x)) * np.sin(x)**2 +\
+                    128.0 * self.s_pm * np.sin(x)**2 * np.sin(0.5*x)**4
+            else:
+                print("PSD option should be in [X, XY, A, E, T] (%s)"%option)
+                return None
+            if tdi2:
+                factor_tdi2 = 4 * np.sin(2 * x)**2
+                s_n *= factor_tdi2
 
-        lisaLT = self.arm_length/CLIGHT
-        x = 2.0 * np.pi * lisaLT * self.freq
-        if option=="X":
-            S = 16.0 * np.sin(x)**2 * (2.0 * (1.0 + np.cos(x)**2) * self.Spm + self.Sop)
-        elif option=="XY":
-            S = -4.0 * np.sin(2*x) * np.sin(x) * (self.Sop + 4.0*self.Spm)
-        elif option in ["A", "E"]:
-            S = 8.0 * np.sin(x)**2 * (2.0 * self.Spm * (3.0 + 2.0*np.cos(x) +\
-                                                        np.cos(2*x)) +\
-                                      self.Sop * (2.0 + np.cos(x)))
-        elif option=="T":
-            S = 16.0 * self.Sop * (1.0 - np.cos(x)) * np.sin(x)**2 +\
-                128.0 * self.Spm * np.sin(x)**2 * np.sin(0.5*x)**4
         else:
-            print("PSD option should be in [X, XY, A, E, T] (%s)"%option)
-            return None
-        if tdi2:
-            factor_tdi2 = 4 * np.sin(2 * x)**2
-            S *= factor_tdi2
+            cov_eta = self._single_link_cov()
+            # TDI transfer function
+            xyz_tf = tdi.compute_tdi_tf(self.freq, self.ltt, tdi2=tdi2, tdi_var='XYZ')
+
+            if option=="X":
+                s_n = tdi.transform_covariance(xyz_tf, cov_eta)[:, 0, 0]
+            elif option == "XY":
+                s_n = tdi.transform_covariance(xyz_tf, cov_eta)[:, 0, 1]
+            elif option in ["A", "E", "T", "AET"]: #transform transfer function into AET
+                aet_tf = tdi.dot(tdi.aet_mat[np.newaxis, :, :], xyz_tf)
+                cov_tdi = tdi.transform_covariance(aet_tf, cov_eta)
+                if option == "A":
+                    s_n = cov_tdi[:, 0, 0]
+                elif option == "E":
+                    s_n = cov_tdi[:, 1, 1]
+                elif option == "T":
+                    s_n = cov_tdi[:, 2, 2]
+                elif option == "AET":
+                    s_n = cov_tdi
+
         if self.wd:
-            S += self.wd_confusion(freq=freq, option=option, tdi2=tdi2)
-        return S
+            s_n += self.wd_confusion(freq=freq, option=option, tdi2=tdi2)
+        return s_n
 
 
 class SciRDdeg1Noise(AnalyticNoise):
-    """ First degration of SciRD 
+    """ First degration of SciRD
     """
     def __init__(self, frq, model="SciRDv1", wd=0):
 
         AnalyticNoise.__init__(self, frq, model, wd=wd)
-        Sa_a = self.DSa_a['SciRDv1'] * (1.0+(0.4e-3/frq)**2) * \
+        Sa_a = self.psd_a_a['SciRDv1'] * (1.0+(0.4e-3/frq)**2) * \
                (1.0+(frq/8e-3)**4) * (1.0+(0.1e-3/frq)**2)
         self.Sa_d = Sa_a*(2.*np.pi*frq)**(-4.)
         Sa_nu = self.Sa_d*(2.0*np.pi*frq/CLIGHT)**2
@@ -331,7 +440,7 @@ class SciRDdeg2Noise(AnalyticNoise):
     def __init__(self, frq, model="SciRDv1", wd=0):
 
         AnalyticNoise.__init__(self, frq, model, wd=wd)
-        Sa_a = self.DSa_a['SciRDv1'] * (1.0+(0.4e-3/frq)**2) * \
+        Sa_a = self.psd_a_a['SciRDv1'] * (1.0+(0.4e-3/frq)**2) * \
                (1.0+(frq/8e-3)**4) * (1.0+(0.1e-3/frq)**4)
         self.Sa_d = Sa_a*(2.*np.pi*frq)**(-4.)
         Sa_nu = self.Sa_d*(2.0*np.pi*frq/CLIGHT)**2
@@ -358,14 +467,14 @@ class LCESAcallNoise(AnalyticNoise):
         AnalyticNoise.__init__(self, frq, wd=wd)
 
         # Acceleration noise
-        Sa_a = self.DSa_a['Proposal'] *\
+        Sa_a = self.psd_a_a['Proposal'] *\
                (1.0 +(0.4e-3/frq)**2+(frq/9.3e-3)**4) # in acceleration
         self.Sa_d = Sa_a*(2.*np.pi*frq)**(-4.) # in displacement
         Sa_nu = self.Sa_d*(2.0*np.pi*frq/CLIGHT)**2 # in relative frequency unit
         self.Spm =  Sa_nu
 
         # Optical Metrology System
-        self.Soms_d = self.DSoms_d['Proposal'] * (1. + (2.e-3/frq)**4) # in displacement
+        self.Soms_d = self.psd_oms_d['Proposal'] * (1. + (2.e-3/frq)**4) # in displacement
         Soms_nu = self.Soms_d*(2.0*np.pi*frq/CLIGHT)**2 # in relative frequency unit
         self.Sop =  Soms_nu
         self.freq = frq
@@ -380,8 +489,8 @@ class NewDRSNoise(AnalyticNoise):
         defaultD = 0.4
         defaultP = 1.0
 
-        lisaD = 0.3  
-        lisaP = 2.0  
+        lisaD = 0.3
+        lisaP = 2.0
         lisaLT = self.arm_length/CLIGHT
         Sops = 6.15e-38 * (lisaLT/defaultL)**2 * (defaultD/lisaD)**4 *\
                (defaultP/lisaP) # 11.83 pm/sqrt(Hz)
@@ -452,7 +561,7 @@ class GalNoise():
         """
         res = self.Ampl*np.exp(-(self.freq/self.fr1)**self.alpha) *\
               (self.freq**(-7./3.))*0.5*(1.0 + np.tanh(-(self.freq-self.fknee)/self.fr2))
-        
+
         return res
 
 
