@@ -13,11 +13,12 @@ from ldc.common.series import FrequencySeries
 from ldc.waveform import imrphenomD
 from ldc.waveform.waveform.hphc import HpHc
 
-YRSID_SI = constants.SIDEREALYEAR_J2000DAY*24*60*60
-MTsun = constants.GM_SUN/constants.SPEED_OF_LIGHT**3
-
 #pylint:disable=E1101
 #pylint:disable=C0103
+#pylint:disable=W0201
+
+YRSID_SI = constants.SIDEREALYEAR_J2000DAY*24*60*60
+MTsun = constants.GM_SUN/constants.SPEED_OF_LIGHT**3
 
 class BHB_IMRPhenomD(HpHc):
     """ Compute waveforms h+ and hx of a black hole binary either massive or of stellar origin.
@@ -36,7 +37,7 @@ class BHB_IMRPhenomD(HpHc):
                      'Tobs':        'ObservationDuration',
                      'dt':          'Cadence',
                      'DL':          lambda p: p['Distance']} #Mpc -> Mpc}
-    
+
     def precomputation(self):
         """ Load required parameters and convert them in expected units. """
 
@@ -45,9 +46,9 @@ class BHB_IMRPhenomD(HpHc):
         if self.source_type.lower() == "mbhb" :
             theL = p['InitialPolarAngleL']
             phiL = p['InitialAzimuthalAngleL']
-            self.pol, self.incl = tools.aziPolAngleL2PsiIncl(self.eclLat, 
-                                                            self.eclLon, 
-                                                            theL, 
+            self.pol, self.incl = tools.aziPolAngleL2PsiIncl(self.eclLat,
+                                                            self.eclLon,
+                                                            theL,
                                                             phiL)
             self.cos2Pol = np.cos(2.*self.pol)
             self.sin2Pol = np.sin(2.*self.pol)
@@ -55,7 +56,7 @@ class BHB_IMRPhenomD(HpHc):
         if self.source_type.lower() == "sbbh":
             #Even though SOBHB are not defined by coalescence time,
             #this parameters allows to use compute_hphc_fd without any rewriting
-            self.tc = 0.0  
+            self.tc = 0.0
             self.incl = p['Inclination']
             self.phi0 = p['InitialPhase']
             self.f0 = p["InitialFrequency"]
@@ -82,8 +83,8 @@ class BHB_IMRPhenomD(HpHc):
                     'Redshift':                 '1',\
                     'Distance':                 'Mpc',
                 }
-        
-        if self.source_type.lower() == "mbhb" :            
+
+        if self.source_type.lower() == "mbhb" :
             BHBunits['CoalescenceTime'] = 's'
             BHBunits['PolarAngleOfSpin1'] = 'rad'
             BHBunits['PolarAngleOfSpin2'] = 'rad'
@@ -91,7 +92,7 @@ class BHB_IMRPhenomD(HpHc):
             BHBunits['InitialPolarAngleL'] = 'rad'
             BHBunits['InitialAzimuthalAngleL'] = 'rad'
 
-        if self.source_type.lower() == "sbbh":            
+        if self.source_type.lower() == "sbbh":
             BHBunits["Inclination"] = 'rad'
             BHBunits["InitialPhase"] = 'rad'
             BHBunits["Polarization"] = 'rad'
@@ -99,6 +100,12 @@ class BHB_IMRPhenomD(HpHc):
 
         return BHBunits
 
+    @property
+    def citation(self):
+        """ Return waveform citation info.
+        """
+        return imrphenomD._citation
+
     def check_param(self):
         """ Check parameters and their units
         """
@@ -108,9 +115,9 @@ class BHB_IMRPhenomD(HpHc):
                 assert k in self.pnames
             except AssertionError:
                 raise AssertionError(f"Field {k} not found in input params")
-                
-        
-        if self.source_type.lower() == "mbhb" :            
+
+
+        if self.source_type.lower() == "mbhb" :
             assert self.units["PhaseAtCoalescence"].lower() in ["radian", "rad", "r"]
             assert self.units["PolarAngleOfSpin1"].lower() in ["radian", "rad", "r"]
             assert self.units["PolarAngleOfSpin2"].lower() in ["radian", "rad", "r"]
@@ -165,36 +172,37 @@ class BHB_IMRPhenomD(HpHc):
         Ms = (self.m1s + self.m2s) * MTsun
         fmax = min(MfCUT_PhenomD/Ms, self.maxf)
         Nf = int((fmax-f0)/self.df)
-        freq_all = np.arange(Nf)*self.df + f0 ### freq. array for waveform 
+        freq_all = np.arange(Nf)*self.df + f0 ### freq. array for waveform
         # Generate the signal
         PhD = imrphenomD.pyIMRPhenomDh22AmpPhase(freq_all)
         frS, ampS, phS = PhD.get_fd_waveform(ph_ref, fRef,m1s_si, m2s_si,
                                              self.a1, self.a2, self.dist)
         return frS, phS, ampS
 
-    def SOBHB_frequency_grid(self,tf,frspl,ind,frS,nptmin) :
-        ### Constructing the reduced freq. array for computing the response f-n
+    def SOBHB_frequency_grid(self, tf, frspl, ind, frS, nptmin) :
+        """ Constructing the reduced freq. array for computing the response f-n.
+        """
         # uniform in time
         Delta_t = 1.0/24.0*constants.SIDEREALYEAR_J2000DAY*86400.
         n_times = int(np.ceil(abs(tf[0] - tf[ind])/Delta_t))+1
         times_d_res = np.linspace(tf[0], tf[ind], n_times)
-    
+
         freqs_d_res = frspl(times_d_res)
         freqs_d_res[0]= self.f0
-        freqs_d_res[-1] = frS[-1]    
+        freqs_d_res[-1] = frS[-1]
         freqs_d_res =  freqs_d_res[freqs_d_res-frS[-1]<=0]
-        
+
         # UNiform in freq.
         Delta_f = 5e-4
         npt_deltafsampling = int(np.ceil((frS[-1] - frS[0])/Delta_f))
         if npt_deltafsampling<=0:freq_rsdeltaf = np.array([])
         else: freq_rsdeltaf = frS[::int(len(frS)/npt_deltafsampling)]
-    
+
         # uniform in log
         freq_rsln = np.logspace(np.log10(self.f0),np.log10(frS[-1]),num=nptmin)
         freq_tmp= np.concatenate((freqs_d_res, freq_rsdeltaf, freq_rsln[1:-1]))
         freq_tmp= np.unique(np.sort(freq_tmp))
-    
+
         ind_fi= np.zeros(len(freq_tmp), dtype='int')
         freq_rs = np.zeros(len(freq_tmp))
         freq_rs[0]= freq_tmp[0]
@@ -210,44 +218,43 @@ class BHB_IMRPhenomD(HpHc):
         ind_fi = ind_fi[ind_u]
         return freq_rs,ind_fi
 
-    def SOBHB_IMRPhenomD_waveform(self, maxf=0.5, settRefAtfRef=True, tRef=0.0, 
-                                    nptmin=1000, MfCUT_PhenomD=0.2-1e-7):
-        """ Return ampl and phase in freq. domain
-        for a SOBHB waveform
+    def SOBHB_IMRPhenomD_waveform(self, maxf=0.5, settRefAtfRef=True, tRef=0.0,
+                                  nptmin=1000):
+        """ Return ampl and phase in freq. domain for a SOBHB waveform
         """
         ## Generate the PhenomD phase and amplitude
         m1_SI = self.m1s*constants.SUN_MASS
-        m2_SI = self.m2s*constants.SUN_MASS 
+        m2_SI = self.m2s*constants.SUN_MASS
         cnt = 2
-        #### Generate PhenomD amplitude and phase on the fine and regular grid 
-        #### of points in freq for efficiency we guess the end frequency and 
+        #### Generate PhenomD amplitude and phase on the fine and regular grid
+        #### of points in freq for efficiency we guess the end frequency and
         #### extend it if not enough
         while cnt <= 50 :                   #TODO : Arbitrary, can be changed
             fmax= min(cnt*self.f0, maxf)
-            if int(np.rint(fmax*self.Tobs)) == 1 : 
+            if int(np.rint(fmax*self.Tobs)) == 1 :
                 raise ValueError("The duration is not sufficient.")
             # freq_fromf0 = np.arange(self.f0,maxf,1./self.Tobs)
             freq_fromf0 = np.arange(self.f0,fmax,1./self.Tobs)
             # Generate the frequency, amplitude and phase of the waveform
             PhD = imrphenomD.pyIMRPhenomDh22AmpPhase(freq_fromf0)
-            fr_a,amp_a,ph_a = PhD.get_fd_waveform(self.phi0, self.f0, 
-                                                    m1_SI, m2_SI, 
-                                                    self.a1, self.a2, 
+            fr_a,amp_a,ph_a = PhD.get_fd_waveform(self.phi0, self.f0,
+                                                    m1_SI, m2_SI,
+                                                    self.a1, self.a2,
                                                     self.dist)
 
             # Spline the data
             tfspline= spline(fr_a, 1/(2.*np.pi)*ph_a).derivative()
-            tf = tfspline(fr_a)    
+            tf = tfspline(fr_a)
             tf -= tf[0]
 
             # Stopping the loop if max duration/frequency is reached
-            if (fmax == maxf) :         #Check if max frequency is reached
+            if fmax == maxf:         #Check if max frequency is reached
                 ind = np.argwhere(np.diff(tf)<0)
                 ind = ind[0][0] if len(ind) else len(tf)-1
                 frspl = spline(tf[:ind], fr_a[:ind])
                 fr_end = fr_a[ind]
                 break
-            if (tf[-1] > 2*self.Tobs) : #Check if timerange is above duration
+            if tf[-1] > 2*self.Tobs: #Check if timerange is above duration
                 ind = np.argwhere(tf > self.Tobs)[0][0]
                 frspl = spline(tf[:ind], fr_a[:ind])
                 fr_end = frspl(self.Tobs)
@@ -267,7 +274,7 @@ class BHB_IMRPhenomD(HpHc):
             phS += 2.*np.pi*(frS - fRef_inrange)*(tRef - tfspline(fRef_inrange))
         return freq_rs,phS[ind_fi],ampS[ind_fi]
 
-    def compute_hphc_fd(self, source_parameters=None, simbuffer=0.):
+    def compute_hphc_fd(self, source_parameters=None, simbuffer=0., tapering=False):
         """ Return hp,hc in the source frame.
 
         Wrapper around MLDC IMRPhenomD model handling Fourier transforms.
@@ -293,33 +300,45 @@ class BHB_IMRPhenomD(HpHc):
         # compute spherical harmonic coeffs associated to 22 mode
         spin_wsh_p = tools.spinWeightedSphericalHarmonic(-2, 2, 2,
                                                         self.incl, self.phi0)
-        spin_wsh_m = tools.spinWeightedSphericalHarmonic(-2, 2, -2, 
+        spin_wsh_m = tools.spinWeightedSphericalHarmonic(-2, 2, -2,
                                                         self.incl, self.phi0)
-        
-        K22p = 1/2.* ( spin_wsh_p + np.conj(spin_wsh_m) )   
-        K22c = 1j/2.*( spin_wsh_p - np.conj(spin_wsh_m) ) 
+
+        K22p = 1/2.* ( spin_wsh_p + np.conj(spin_wsh_m) )
+        K22c = 1j/2.*( spin_wsh_p - np.conj(spin_wsh_m) )
 
         #Phase and timeshift if necessary
         t0 = -self.tc - simbuffer # shift the data by tc
         phasetimeshift = -2.0*np.pi*frS*t0
-        CommonPart = ampS * np.exp(1.0j * (phS + phasetimeshift)) ### 22 mode
+        CommonPart = ampS * np.exp(1.0j * (phS))# + phasetimeshift)) ### 22 mode
+        ExtraPart = np.exp(1.0j * (phasetimeshift))
+
+        #Tapering before ifft
+        if tapering:
+            deltaminf, deltamaxf = 1e-5, 4e-3
+            minf = tools.newtonianfoft(tools.mchirpofm1m2(self.m1s, self.m2s),
+                                       self.Tobs/YRSID_SI)
+            maxf = 5e-2
+            w = tools.window_planck_vec(frS, minf, maxf, deltaminf, deltamaxf)
+            CommonPart = w * CommonPart
+
+        i_b = int(np.rint(frS[0]/self.df))
         del ampS, phS, phasetimeshift
 
         # creating full arrays (from zero freq, to nyquist)
-        i_b = int(np.rint(frS[0]/self.df))
         del frS
-        hp_f = FrequencySeries(np.conjugate(K22p * CommonPart),
-                               df=self.df, kmin=i_b)
-        hc_f = FrequencySeries(np.conjugate(K22c * CommonPart),
-                               df=self.df, kmin=i_b)
+        hp_f = FrequencySeries(np.conjugate(K22p * CommonPart * ExtraPart),
+                              df=self.df, kmin=i_b)
+        hc_f = FrequencySeries(np.conjugate(K22c * CommonPart * ExtraPart),
+                              df=self.df, kmin=i_b)
+
         return hp_f, hc_f
 
-    def compute_hphc_td(self, t, source_parameters=None, approx_t=False, 
-                        set_attr=True, simbuffer=0., low_pass=False):
+    def compute_hphc_td(self, t, source_parameters=None, approx_t=False,
+                        set_attr=True, simbuffer=0., low_pass=False, tapering=True):
         """ Return hp, hx for a time samples in t.
 
         Source parameters can be updated at the same time.
-        
+
         >>> MBHB = HpHc.type("my-mbhb", "MBHB", "IMRPhenomD")
         >>> hp_m,hc_m = MBHB.compute_hphc_td(np.arange(0,100,10), pMBHB)
         >>> SOBHB = HpHc.type("my-sobhb", "SBBH", "IMRPhenomD")
@@ -328,10 +347,12 @@ class BHB_IMRPhenomD(HpHc):
         # Check the approximant and call appropriate function
         if 'IMRPhenom' in self.approximant and self.source_type.lower() == "mbhb" :
             hpS, hcS = self.compute_hphc_fd(source_parameters=source_parameters,
-                                            simbuffer=simbuffer)
+                                            simbuffer=simbuffer, tapering=tapering)
+
             # iFFT
             hpS = hpS.ts.ifft(dt=self.dt)
             hcS = hcS.ts.ifft(dt=self.dt)
+
         elif 'IMRPhenom' in self.approximant and self.source_type.lower() == "sbbh" :
 
             #Ensure that all parameters are well written
@@ -358,7 +379,7 @@ class BHB_IMRPhenomD(HpHc):
 
             # interpolation function from inverse SPA timeline to wanted timeline
             spl_amp = spline(tspa, amp)
-            spl_ph = spline(tspa, ph)          
+            spl_ph = spline(tspa, ph)
 
             ### NOK for phase
             ph_at0 = spl_ph(0.0)
@@ -417,8 +438,8 @@ if __name__ == "__main__":
     import doctest
     pMBHB = dict({'Mass1':                  132628.202,
                   'Mass2':                  30997.2481,
-                  'Spin1':                  0.9481998052314212, 
-                  'Spin2':                  0.9871324769575264, 
+                  'Spin1':                  0.9481998052314212,
+                  'Spin2':                  0.9871324769575264,
                   'EclipticLatitude':       0.312414,
                   'EclipticLongitude':      -2.75291,
                   'Redshift':               2.0,
