@@ -77,8 +77,8 @@ class TDIFactory():
     def get_default_param(self):
         if self.approx == "TD_fdot":
             return dict({'Amplitude': 1.07345e-22,
-                         'EclipticLatitude': np.pi, #0.312414*un.rad,
-                         'EclipticLongitude': np.pi/2., #-2.75291*un.rad,
+                         'EclipticLatitude': 0.312414*un.rad, #np.pi, #
+                         'EclipticLongitude': -2.75291*un.rad, #np.pi/2., #,
                          'Frequency': 0.00135962*un.Hz,
                          'FrequencyDerivative': 8.94581279e-19*un.Unit('Hz2'),
                          'Inclination': 0.523599*un.rad,
@@ -342,7 +342,7 @@ if __name__ == '__main__':
     if 1: # test MBHB
         facGB = TDIFactory(source_type="MBHB", approximant='IMRPhenomD', dt=5, duration=60*60*24*365)
         ldc = facGB.ldc(tt_order=0)
-        ldc1 = facGB.ldc(tt_order=1)
+        #ldc1 = facGB.ldc(tt_order=1)
         fast = facGB.fast()
 
         plt.figure(figsize=(15,10))
@@ -353,7 +353,7 @@ if __name__ == '__main__':
         plt.axis([facGB.param["CoalescenceTime"]-1000,
                   facGB.param["CoalescenceTime"]+500, None, None])
         ldc_s = ldc.sel(t=fast.t, method="nearest")
-        ldc_s1 = ldc1.sel(t=fast.t, method="nearest")
+        #ldc_s1 = ldc1.sel(t=fast.t, method="nearest")
         plt.subplot(212)
         plt.semilogy(fast.t, np.abs(ldc_s.data-fast.X.data), label="ldc00-fast")
         plt.legend()
@@ -364,8 +364,8 @@ if __name__ == '__main__':
         plt.figure(figsize=(15,10))
         plt.subplot(111)
         plt.semilogy(fast.t, np.abs(ldc_s.data-fast.X.data), label="ldc00-fast")
-        plt.semilogy(fast.t, np.abs(ldc_s1.data-fast.X.data), label="ldc11-fast")
-        plt.semilogy(fast.t, np.abs(ldc_s1.data-ldc_s.data), label="ldc11-ldc00")
+        #plt.semilogy(fast.t, np.abs(ldc_s1.data-fast.X.data), label="ldc11-fast")
+        #plt.semilogy(fast.t, np.abs(ldc_s1.data-ldc_s.data), label="ldc11-ldc00")
         plt.legend()
         plt.grid()
         plt.axis([facGB.param["CoalescenceTime"]-1000,
